package com.zy.bms.common.aspect;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zy.bms.common.ServerResponse;
import com.zy.bms.common.enums.ResponseCode;
import com.zy.bms.entity.DeviceBase;
import com.zy.bms.entity.UserPrivilege;
import com.zy.bms.service.IDeviceBaseService;
import com.zy.bms.service.IUserPrivilegeService;
import com.zy.bms.utils.AesUtil;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.CodeSignature;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;

/**
 * 微信用户权限拦截
 */
@Slf4j
//@Aspect
//@Component
public class UserPrivilegeWxAop {

    @Resource
    private IUserPrivilegeService userPrivilegeService;

    @Resource
    private IDeviceBaseService deviceBaseService;


    @Around(value = "@annotation(com.zy.bms.common.annotation.UserPrivilegeWx)")
    public Object doAround(ProceedingJoinPoint joinPoint) {
        try {
            //获取request对象
            HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            String userId = AesUtil.decrypt(request.getHeader("app_user"));
            HashMap<String, Object> params = getNameAndValue(joinPoint);
            String openNum = params.get("openNum").toString();
            DeviceBase deviceBase = deviceBaseService.getOne(new QueryWrapper<DeviceBase>().eq("open_num", openNum));
            if (deviceBase != null) {
                UserPrivilege userPrivilege = userPrivilegeService
                        .getOne(new QueryWrapper<UserPrivilege>().eq("user_id", userId)
                                .eq("group_id", deviceBase.getGroupId()));
                if (userPrivilege != null) return joinPoint.proceed();
            }
        } catch (Throwable throwable) {
            log.error("微信用户设备权限AOP错误", throwable);
            throwable.printStackTrace();
        }
        return ServerResponse.custom(ResponseCode.AUTHOR);
    }

    /**
     * 获取参数Map
     */
    private HashMap<String, Object> getNameAndValue(ProceedingJoinPoint joinPoint) {
        HashMap<String, Object> result = new HashMap<>();
        Object[] paramValues = joinPoint.getArgs();
        String[] paramNames = ((CodeSignature) joinPoint.getSignature()).getParameterNames();
        for (int i = 0; i < paramNames.length; i++) {
            result.put(paramNames[i], paramValues[i]);
        }
        return result;
    }
}