package com.zy.bms.sender;

import com.zy.bms.common.Constant;
import com.zy.bms.config.mqtt.MqttGateway;
import com.zy.bms.entity.DeviceBase;
import com.zy.bms.handler.WebSocketHandler;
import com.zy.bms.pojo.dto.MqttDTO;
import com.zy.bms.service.impl.LogsService;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * Mqtt消息发送者
 *
 * @author yang xiao kun
 * create on 2021/10/26
 */
@Component
public class BaseSender {

    @Resource
    private MqttGateway mqttGateway;
    @Resource
    private LogsService logsService;
    @Resource
    private WebSocketHandler webSocketHandler;

    /**
     * 发送Mqtt 消息
     *
     * @param topic   主题
     * @param payload 内容
     */
    public void send(String topic, String payload) {
        // 1. 发送 MQTT 协议消息
        mqttGateway.sendMsgToMqtt(payload, topic);
        // 2. 存储消息日志
        logsService.saveSend(topic, payload);
        // 3. 消息同步转发至webSocket
        webSocketHandler.massMessage(Constant.splitOpenNum(topic), payload);
    }


}
