package com.zy.bms.sender;

import com.zy.bms.common.Constant;
import com.zy.bms.pojo.dto.MqttDTO;
import com.zy.bms.pojo.io.set.AlarmIO;
import com.zy.bms.pojo.io.set.NtripIO;
import com.zy.bms.pojo.io.set.SetIO;
import com.zy.bms.pojo.io.set.TcpIO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * 普适型
 * 发送 MQTT 消息
 *
 * @author yang xiao kun
 * create on 2021/10/26
 */
@Slf4j
@Component
public class UbiSender extends BaseSender {

    public void set(SetIO io) {
        //发送消息
        send(Constant.TOPIC.DEVICE_CLIENT + io.getOpenNum(), getMqttDTO(io).json());
    }

    /**
     * 请求设备远程设置
     *
     * @param openNum 设备码
     */
    public void requestRemoteSet(String openNum) {
        MqttDTO mqttDTO = new MqttDTO(Constant.M.UBI_GET_SET_INFO);
        mqttDTO.put("getset", "getset");
        send(Constant.TOPIC.DEVICE_CLIENT + openNum, mqttDTO.json());
    }


    /**
     * 请求设备立即发送心跳包
     *
     * @param openNum 设备码
     */
    public void requestHeartbeat(String openNum) {
        MqttDTO mqttDTO = new MqttDTO(Constant.M.UBI_REQUEST_HEARTBEAT);
        mqttDTO.put("fun", "gethearbeat");
        send(Constant.TOPIC.DEVICE_CLIENT + openNum, mqttDTO.json());
    }

    /**
     * 请求设备蜂鸣器
     *
     * @param openNum 设备码
     */
    public void requestBuzzing(String openNum) {
        MqttDTO mqttDTO = new MqttDTO(Constant.M.UBI_REQUEST_HEARTBEAT);
        mqttDTO.put("fun", "buzzerring");
        send(Constant.TOPIC.DEVICE_CLIENT + openNum, mqttDTO.json());
    }

    /**
     * 组装需要发送的模板消息
     *
     * @param io 设置信息
     */
    private MqttDTO getMqttDTO(SetIO io) {
        MqttDTO result = new MqttDTO(io.getM());
        switch (io.getM()) {
            //普适性 - ntripServer设置
            case Constant.M.UBI_NTRIP_SERVER: {
                NtripIO tmp = (NtripIO) io;
                result.put("en", tmp.getOnOff() == 0 ? "off" : "on").put("host", tmp.getHost())
                        .put("port", tmp.getPort()).put("mountpoint", tmp.getMountPoint())
                        .put("password", tmp.getPassword());
                break;
            }
            //普适性 - ntripClient设置
            case Constant.M.UBI_NTRIP_CLIENT: {
                NtripIO tmp = (NtripIO) io;
                result.put("en", tmp.getOnOff() == 0 ? "off" : "on").put("host", tmp.getHost())
                        .put("port", tmp.getPort()).put("mountpoint", tmp.getMountPoint())
                        .put("password", tmp.getPassword()).put("user", tmp.getUser());
                break;
            }
            //普适性 - tcp设置
            case Constant.M.UBI_TCP: {
                TcpIO tmp = (TcpIO) io;
                result.put("en", tmp.getOnOff() == 0 ? "off" : "on").put("host", tmp.getHost())
                        .put("port", tmp.getPort()).put("type", tmp.getType());
                break;
            }
            //普适性 - 远程控制设备报警
            case Constant.M.UBI_CONTROL_ALARM: {
                AlarmIO tmp = (AlarmIO) io;
                result.put("duration", tmp.getDuration()).put("freq", tmp.getFreq())
                        .put("times", tmp.getTimes());
                break;
            }
        }
        return result;
    }

}
