package com.zy.bms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zy.bms.entity.UserPrivilege;
import com.zy.bms.mapper.UserPrivilegeMapper;
import com.zy.bms.pojo.vo.GroupListVo;
import com.zy.bms.service.IGroupService;
import com.zy.bms.service.IUserPrivilegeService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 用户查看手机设备权限表 服务实现类
 *
 * @author chenyi
 * @since 2021-07-13
 */
@Service
public class UserPrivilegeServiceImpl extends ServiceImpl<UserPrivilegeMapper, UserPrivilege> implements IUserPrivilegeService {

    @Resource
    private IGroupService groupService;


    @Override
    public List<String> getGroupIdsByUserId(Integer userId) {
        return baseMapper.getGroupIdsByUserId(userId);
    }

    @Override
    public Map<String, List<GroupListVo>> getNoAccessGroups(Integer userId) {
        //全部的设备组
        List<GroupListVo> allGroup = groupService.getAllList();
        //用户已拥有权限的设备组ID
        HashSet<String> ownGroup = new HashSet<>(getGroupIdsByUserId(userId));
        //过滤掉已拥有权限的设备组后按项目ID分组
        return allGroup.stream().filter(item -> !ownGroup.contains(item.getCode()))
                .collect(Collectors.groupingBy(GroupListVo::getItem));
    }

    @Override
    public boolean hasPrivilege(Integer userId, String groupId) {
        return baseMapper.selectOne(new QueryWrapper<UserPrivilege>()
                .eq("group_id", groupId)
                .eq("user_id", userId)) != null;
    }

    @Override
    public void saveBatch(Integer userId, String[] groups) {
        baseMapper.saveBatch(userId, groups);
    }
}
